//+------------------------------------------------------------------+
//|                                   Double Smoothed Stochastic.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_minimum 0
#property indicator_maximum 1
#property indicator_width1  2
#property indicator_width2  2

extern int  StochasticPeriod = 55;
extern int  EMAPeriod        = 15;
extern bool alertsOn         = false;
extern bool alertsOnCurrent  = true;
extern bool alertsMessage    = true;
extern bool alertsSound      = false;
extern bool alertsEmail      = false;

double up[];
double dn[];
double dss[];
double slope[];
double wr[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0, up); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1, dn); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2, dss);
   SetIndexBuffer(3, wr);
   SetIndexBuffer(4, slope);
      IndicatorShortName("DSS");//("Double smoothed stochastic "+StochasticPeriod+","+EMAPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   double max,min,sto;
   double alpha = 2.0/(1.0+EMAPeriod);
   int    counted_bars=IndicatorCounted();
   int    i,limit;


   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars - counted_bars,Bars-1);

         
      for(i = limit; i >= 0 ; i--)
      {
         max = High[ArrayMaximum(High,StochasticPeriod,i)];
         min =  Low[ArrayMinimum( Low,StochasticPeriod,i)];
         if (max!=min)
               sto = (Close[i]-min)/(max-min)*100.00;
         else  sto = 0;            
         wr[i] = wr[i+1]+alpha*(sto-wr[i+1]);
           
         max = wr[ArrayMaximum(wr,StochasticPeriod,i)];
         min = wr[ArrayMinimum(wr,StochasticPeriod,i)];
         if (max!=min)
               sto = (wr[i]-min)/(max-min)*100.00;
         else  sto = 0;            
         dn[i]    = EMPTY_VALUE;
         up[i]    = EMPTY_VALUE;
         dss[i]   = dss[i+1]+alpha*(sto-dss[i+1]);
         slope[i] = slope[i+1];
            if (dss[i]>dss[i+1]) slope[i] =  1;
            if (dss[i]<dss[i+1]) slope[i] = -1;
            if (slope[i] ==  1) up[i] =  1;
            if (slope[i] == -1) dn[i] =  1;
   }
   manageAlerts();
   return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (slope[whichBar] != slope[whichBar+1])
      {
         if (slope[whichBar] ==  1) doAlert("up");
         if (slope[whichBar] == -1) doAlert("down");
      }
   }
}

//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message = Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" DSS slope (trend) changed to "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," DSS "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}


